#!/usr/bin/env python
# coding: utf-8
"""
Module to create MIDI events.
"""

def notesToMidiString(l):
    """
    Set the lyrics on the NSX-39

    You can set a single note, or a series of notes.
    The device is capable of holding a string of lyrics
    that it will sequence through as notes are played.
    You can update the lyrics at any time.
    """
    output = "F0 43 79 09 11 0A 00 "
    output += " ".join(["%2x" % i for i in l])
    output += " F7"
    return output

def noteOn(note, velocity=0x45, channel='0'):
    """
    Create a MIDI noteon event.

    note is required
    velocity and channel are optional
    channel should be a hex digit character
    """
    return "9%c %2x %2x" % (channel, note, velocity)

def noteOff(note, velocity=0x45, channel='0'):
    """
    Create a MIDI noteoff event.

    note is required
    velocity and channel are optional
    channel should be a hex digit character
    """
    return "8%c %2x %2x" % (channel, note, velocity)

def aftertouch(note, velocity=0x45, channel='0'):
    """
    Create a MIDI aftertouch event.

    note is required
    velocity and channel are optional
    channel should be a hex digit character
    """
    return "A%c %2x %2x" % (channel, note, velocity)

def bend(lsb, msb, channel='0'):
    """
    Create a MIDI pitchbend event

    lsb and msb and required
    channel is optional and should be a hex digit character
    """
    return "E%c %2x %2x" % (channel, lsb, msb)




#!/usr/bin/env python
# coding: utf-8
"""
Table of Japanese mora to note values for the NSX-1 chip,
as well as helpful functions to deal with the table.
"""

characters = [u'あ', u'い', u'う', u'え', u'お', u'か', u'き', u'く', u'け', u'こ', u'が', u'ぎ', u'ぐ', u'げ', u'ご', u'きゃ', u'きゅ', u'きょ', u'ぎゃ', u'ぎゅ', u'ぎょ', u'さ', u'すぃ', u'す', u'せ', u'そ', u'ざ', u'ずぃ', u'ず', u'ぜ', u'ぞ', u'しゃ', u'し', u'しゅ', u'しぇ', u'しょ', u'じゃ', u'じ', u'じゅ', u'じぇ', u'じょ', u'た', u'てぃ', u'とぅ', u'て', u'と', u'だ', u'でぃ', u'どぅ', u'で', u'ど', u'てゅ', u'でゅ', u'ちゃ', u'ち', u'ちゅ', u'ちぇ', u'ちょ', u'つぁ', u'つぃ', u'つ', u'つぇ', u'つぉ', u'な', u'に', u'ぬ', u'ね', u'の', u'にゃ', u'にゅ', u'にょ', u'は', u'ひ', u'ふ', u'へ', u'ほ', u'ば', u'び', u'ぶ', u'べ', u'ぼ', u'ぱ', u'ぴ', u'ぷ', u'ぺ', u'ぽ', u'ひゃ', u'ひゅ', u'ひょ', u'びゃ', u'びゅ', u'びょ', u'ぴゃ', u'ぴゅ', u'ぴょ', u'ふぁ', u'ふぃ', u'ふゅ', u'ふぇ', u'ふぉ', u'ま', u'み', u'む', u'め', u'も', u'みゃ', u'みゅ', u'みょ', u'や', u'ゆ', u'よ', u'ら', u'り', u'る', u'れ', u'ろ', u'りゃ', u'りゅ', u'りょ', u'わ', u'うぃ', u'うぇ', u'うぉ', u'N\\', u'm', u'N', u'J', u'n']

aliases = {
    u'づぁ': u'ざ',
    u'づぃ': u'ずぃ',
    u'づ': u'ず',
    u'づぇ': u'ぜ',
    u'づぉ': u'ぞ',
    u'ゐ': u'うぃ',
    u'ゑ': u'うぇ',
    u'を': u'うぉ',
    u'ん': u'n', # XXX
}

def characterToNote(c):
    """Convert a single note to its NSX-1 value."""
    if c in aliases:
        c = aliases[c]
    if c in characters:
        return characters.index(c)
    raise ValueError("Invalid note character: %s" % c)

def characterIsValid(c):
    """Determine if a character string represents a valid note mora."""
    if c in aliases:
        c = aliases[c]
    return c in characters

def stringToNotes(s):
    """Convert a string of notes to a list of NSX-1 values."""
    i = 0
    output = []
    while i < len(s):
        candidate = s[i:i+2]
        if characterIsValid(candidate):
            output.append(characterToNote(candidate))
            i += 2
        else:
            candidate = s[i]
            if characterIsValid(candidate):
                output.append(characterToNote(candidate))
            i += 1
    return output


if __name__ == "__main__":
    # Tests
    assert characters[0x00] == u'あ'
    assert characters[0x43] == u'の'
    assert characters[0x68] == u'も'
    assert characters[0x7F] == u'n'

    assert characterToMidi(u'ば') == 0x4C
    assert characterToMidi(u'り') == 0x70

    assert stringToNotes(u"でぃおすみおえすらちゅぱかぶら") == \
            [0x2F, 0x04, 0x17, 0x65, 0x04, 0x03, 0x17, 0x6F, 0x37, 0x51, 0x05, 0x4E, 0x6F]



#!/usr/bin/env python
# coding: utf-8
"""
Module to handle interaction with amidi.
"""

from subprocess import call, check_output

hardware_port = None

def detectHardwarePort():
    """Automatically detect the NSX-39 port."""
    global hardware_port
    devices = check_output(['amidi', '-l'])
    for line in devices.split('\n'):
        if line.startswith('IO') and "NSX-39" in line:
            hw = line[line.index('hw'):]
            hw = hw[:hw.index(' ')]
            hardware_port = hw
            return
    raise ValueError("Couldn't find an NSX-39 T-T")

def send(s):
    """Send a raw MIDI string to the NSX-39."""
    if not hardware_port:
        detectHardwarePort()
    call(['amidi', '-p', hardware_port, '-S', s])




from setuptools import setup, find_packages
import codecs
import os
import re

here = os.path.abspath(os.path.dirname(__file__))

def find_version(*file_paths):
    with codecs.open(os.path.join(here, *file_paths), 'r', 'latin1') as f:
        version_file = f.read()
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")

setup(
    name="pocketmiku",
    version=find_version('pocketmiku', '__init__.py'),
    description="Tools to configure a Pocket Miku (NSX-39)",
    long_description="""This package provides tools to set the lyrics on and send MIDI events to a Pocket Miku (NSX-39) hardware voice synthesizer.""",

    url='https://github.com/klange/pocketmiku',

    author='Kevin Lange',
    author_email='klange@dakko.us',

    license='University of Illinois/NCSA Open Source License',

    classifiers=[
        'Development Status :: 4 - Beta',
        'Topic :: Multimedia :: Sound/Audio :: MIDI',
        'Topic :: Multimedia :: Sound/Audio :: Sound Synthesis',
        'Topic :: Multimedia :: Sound/Audio :: Speech',
        'License :: OSI Approved :: University of Illinois/NCSA Open Source License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
    ],

    keywords='miku vocaloid pocketmiku nsx-39 midi audio',

    packages=['pocketmiku'],
    install_requires = [],
)

